"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const misc_utils_of_mine_generic_1 = require("misc-utils-of-mine-generic");
const File_1 = require("../File");
/**
 * Creates an CV ImageData object from given image.
 */
function toImageData(img) {
    return {
        data: new Uint8ClampedArray(img.data),
        width: img.cols,
        height: img.rows
    };
}
exports.toImageData = toImageData;
exports.asImageData = toImageData;
function isMat(m) {
    return m && typeof m.rows === 'number' && typeof m.cols === 'number' && typeof m.data === 'object' && typeof m.copyTo === 'function';
}
exports.isMat = isMat;
/**
 * Returns a new image that is identical to given (1, 3 or 4 channels) but has 4 RGBA channels.
 */
function toRgba(mat, dst = new cv.Mat()) {
    const depth = mat.type() % 8;
    const scale = depth <= cv.CV_8S ? 1.0 : (depth <= cv.CV_32S ? 1.0 / 256.0 : 255.0);
    const shift = (depth === cv.CV_8S || depth === cv.CV_16S) ? 128.0 : 0.0;
    mat.convertTo(dst, cv.CV_8U, scale, shift);
    switch (dst.type()) {
        case cv.CV_8UC1:
            cv.cvtColor(dst, dst, cv.COLOR_GRAY2RGBA);
            break;
        case cv.CV_8UC3:
            cv.cvtColor(dst, dst, cv.COLOR_RGB2RGBA);
            break;
        case cv.CV_8UC4:
            break;
        default:
            throw new Error('Bad number of channels (Source image must have 1, 3 or 4 channels)');
    }
    return dst;
}
exports.toRgba = toRgba;
function fromFile(f) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = yield File_1.File.fromFile(f);
        return file.asMat();
    });
}
exports.fromFile = fromFile;
function fromArrayBuffer(a) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = yield File_1.File.fromArrayBuffer(a);
        return file.asMat();
    });
}
exports.fromArrayBuffer = fromArrayBuffer;
function fromUrl(f) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = yield File_1.File.fromUrl(f);
        return file.asMat();
    });
}
exports.fromUrl = fromUrl;
/**
 * Compare two images by getting the L2 error (square-root of sum of squared error). The lower the result the more similar are the images.
 */
function compareL2(f1, f2, destroyBoth = false) {
    const a = asMat(f1), b = asMat(f2);
    if (a.rows > 0 && a.rows == b.rows && a.cols > 0 && a.cols == a.cols) {
        // Calculate the L2 relative error between images.
        const errorL2 = cv.norm1(a, b, cv.NORM_L2);
        // Convert to a reasonable scale, since L2 error is summed across all pixels of the image.
        const similarity = errorL2 / (a.rows * a.cols);
        destroyBoth && del(a, b);
        return similarity;
    }
    else {
        //Images have a different size
        return 1.0;
    }
}
exports.compareL2 = compareL2;
function asMat(f) {
    return File_1.File.isFile(f) ? f.asMat() : f;
}
exports.asMat = asMat;
function del(...m) {
    m.forEach(m => misc_utils_of_mine_generic_1.tryTo(() => m.delete()));
}
exports.del = del;
/**
 * sets the color of pixel at coords (x,y)
 */
function set(m, x, y, val) {
    const c = m.channels();
    let view = m.data;
    for (let i = 0; i < val.length; i++) {
        view[y * c * m.cols + x * c + i] = val[i];
    }
}
exports.set = set;
/**
 * gets the color of pixel at coords (x,y)
 */
function get(m, x, y) {
    const c = m.channels();
    let view = m.data;
    const v = [];
    for (let i = 0; i < c; i++) {
        v.push(view[y * c * m.cols + x * c + i]);
    }
    return v;
}
exports.get = get;
function map(mat, dst, fn) {
    for (let y = 0; y < mat.rows; y++) {
        for (let x = 0; x < mat.cols; x++) {
            const v = fn(get(mat, x, y), x, y);
            set(dst, x, y, v);
        }
    }
}
exports.map = map;
let _noArray;
/**
 * for overload methods that won't accept undefined as argument, like 'mask' cv.add()
 */
function noArray() {
    if (!_noArray) {
        _noArray = cv.Mat.ones(0, 0, cv.CV_8U);
    }
    return _noArray;
}
exports.noArray = noArray;
function pointToSize(p) {
    return new cv.Size(p.x, p.y);
}
exports.pointToSize = pointToSize;
function sizeToPoint(s) {
    return new cv.Point(s.width, s.height);
}
exports.sizeToPoint = sizeToPoint;
function isSize(size) {
    return typeof size === 'object' && typeof size.width === 'number';
}
exports.isSize = isSize;
function mat2data(m) {
    return {
        rows: m.rows,
        cols: m.cols,
        type: m.type(),
        data: ab2str(m.data)
    };
}
exports.mat2data = mat2data;
function data2mat(d) {
    return cv.matFromArray(d.rows, d.cols, d.type, str2ab(d.data));
}
exports.data2mat = data2mat;
function isMatData(d) {
    return d && typeof d === 'object' && typeof d.rows === 'number' && typeof d.cols === 'number' && typeof d.type !== 'undefined' && Object.keys(d).sort().join(',') === 'cols,data,rows,type';
}
exports.isMatData = isMatData;
let _Buffer = require('buffer/').Buffer;
if (misc_utils_of_mine_generic_1.isNode() && typeof _Buffer !== 'undefined') {
    _Buffer = Buffer;
}
function ab2str(buf) {
    return _Buffer.from(buf).toString('base64');
}
function str2ab(str) {
    return new Uint8Array(_Buffer.from(str, 'base64'));
}
/**
 * Executes JSON.stringify on JSON containing Mat instances.
 */
function jsonStringifyWithMat(s) {
    return JSON.stringify(s, (key, value) => {
        if (isMat(value)) {
            return mat2data(value);
        }
        else {
            return value;
        }
    });
}
exports.jsonStringifyWithMat = jsonStringifyWithMat;
/**
 * Executes JSON.parse on JSON containing Mat instances.
 */
function jsonParseWithMat(d) {
    return JSON.parse(d, (key, value) => {
        if (isMatData(value)) {
            return data2mat(value);
        }
        else {
            return value;
        }
    });
}
exports.jsonParseWithMat = jsonParseWithMat;
//# sourceMappingURL=imageUtil.js.map